@@ -1,13 +1,13 @@
 import logging
 import time
-from typing import (Any, Dict, Iterable, Iterator, List, Optional, Protocol,
-                    Tuple, Union)
+from typing import (
+    Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union, Callable, cast)
 
 import tqdm
 from opentelemetry import trace
 
 from datapipe.compute import (Catalog, ComputeStep, DatatableTransformStep,
-                              PipelineStep)
+                              PipelineStep, DatatableTransformFunc)
 from datapipe.datatable import DataStore, DataTable
 from datapipe.run_config import RunConfig
 from datapipe.types import ChangeList, DataDF, IndexDF
@@ -16,11 +16,7 @@ logger = logging.getLogger('datapipe.core_steps')
 tracer = trace.get_tracer("datapipe.core_steps")
 
 
-class BatchTransformFunc(Protocol):
-    __name__: str
-
-    def __call__(self, *inputs, **kwargs) -> Union[DataDF, List[DataDF], Tuple[DataDF, ...]]:
-        ...
+BatchTransformFunc = Callable[..., Union[DataDF, List[DataDF], Tuple[DataDF, ...]]]
 
 
 def do_batch_transform(
@@ -29,9 +25,9 @@ def do_batch_transform(
     input_dts: List[DataTable],
     output_dts: List[DataTable],
     idx_gen: Iterable[IndexDF],
-    idx_count: int = None,
-    kwargs: Dict[str, Any] = None,
-    run_config: RunConfig = None,
+    idx_count: Optional[int] = None,
+    kwargs: Optional[Dict[str, Any]] = None,
+    run_config: Optional[RunConfig] = None,
 ) -> Iterator[ChangeList]:
     '''
     Множественная инкрементальная обработка `input_dts' на основе изменяющихся индексов
@@ -48,7 +44,16 @@ def do_batch_transform(
             logger.debug(f'Idx to process: {idx.to_records()}')
 
             with tracer.start_as_current_span("get input data"):
-                input_dfs = [inp.get_data(idx) for inp in input_dts]
+                try:
+                    input_dfs = [inp.get_data(idx) for inp in input_dts]
+                except Exception as e:
+                    logger.error(f"Get input data failed: {str(e)}")
+                    ds.event_logger.log_exception(
+                        e,
+                        run_config=RunConfig.add_labels(run_config, {'idx': idx.to_dict(orient="records")})
+                    )
+
+                    continue
 
             changes = ChangeList()
 
@@ -58,7 +63,10 @@ def do_batch_transform(
                         chunks_df = func(*input_dfs, **kwargs or {})
                     except Exception as e:
                         logger.error(f"Transform failed ({func.__name__}): {str(e)}")
-                        ds.event_logger.log_exception(e, run_config=run_config)
+                        ds.event_logger.log_exception(
+                            e,
+                            run_config=RunConfig.add_labels(run_config, {'idx': idx.to_dict(orient="records")})
+                        )
 
                         continue
 
@@ -69,16 +77,25 @@ def do_batch_transform(
                     chunks_df = [chunks_df]
 
                 with tracer.start_as_current_span("store output batch"):
-                    for k, res_dt in enumerate(output_dts):
-                        # Берем k-ое значение функции для k-ой таблички
-                        # Добавляем результат в результирующие чанки
-                        change_idx = res_dt.store_chunk(
-                            data_df=chunks_df[k],
-                            processed_idx=idx,
-                            run_config=run_config,
+                    try:
+                        for k, res_dt in enumerate(output_dts):
+                            # Берем k-ое значение функции для k-ой таблички
+                            # Добавляем результат в результирующие чанки
+                            change_idx = res_dt.store_chunk(
+                                data_df=chunks_df[k],
+                                processed_idx=idx,
+                                run_config=run_config,
+                            )
+
+                            changes.append(res_dt.name, change_idx)
+                    except Exception as e:
+                        logger.error(f"Store output batch failed: {str(e)}")
+                        ds.event_logger.log_exception(
+                            e,
+                            run_config=RunConfig.add_labels(run_config, {'idx': idx.to_dict(orient="records")})
                         )
 
-                        changes.append(res_dt.name, change_idx)
+                        continue
 
             else:
                 with tracer.start_as_current_span("delete missing data from output"):
@@ -97,9 +114,9 @@ def do_full_batch_transform(
     ds: DataStore,
     input_dts: List[DataTable],
     output_dts: List[DataTable],
-    kwargs: Optional[Dict] = None,
+    kwargs: Optional[Dict[str, Any]] = None,
     chunk_size: int = 1000,
-    run_config: RunConfig = None,
+    run_config: Optional[RunConfig] = None,
 ) -> None:
     with tracer.start_as_current_span("compute ids to process"):
         idx_count, idx_gen = ds.get_full_process_ids(
@@ -131,7 +148,7 @@ class BatchTransform(PipelineStep):
         inputs: List[str],
         outputs: List[str],
         chunk_size: int = 1000,
-        kwargs: Dict[str, Any] = None,
+        kwargs: Optional[Dict[str, Any]] = None,
     ):
         self.func = func
         self.inputs = inputs
@@ -163,7 +180,7 @@ class BatchTransformStep(ComputeStep):
         output_dts: List[DataTable],
 
         func: BatchTransformFunc,
-        kwargs: Dict[str, Any] = None,
+        kwargs: Optional[Dict[str, Any]] = None,
         chunk_size: int = 1000,
     ) -> None:
         ComputeStep.__init__(self, name)
@@ -181,15 +198,16 @@ class BatchTransformStep(ComputeStep):
     def get_output_dts(self) -> List[DataTable]:
         return self.output_dts
 
-    def run_full(self, ds: DataStore, run_config: RunConfig = None) -> None:
+    def run_full(self, ds: DataStore, run_config: Optional[RunConfig] = None) -> None:
         run_config = RunConfig.add_labels(run_config, {'step_name': self.name})
 
-        idx_count, idx_gen = ds.get_full_process_ids(
-            inputs=self.input_dts,
-            outputs=self.output_dts,
-            chunk_size=self.chunk_size,
-            run_config=run_config
-        )
+        with tracer.start_as_current_span("Get ids to process"):
+            idx_count, idx_gen = ds.get_full_process_ids(
+                inputs=self.input_dts,
+                outputs=self.output_dts,
+                chunk_size=self.chunk_size,
+                run_config=run_config
+            )
 
         gen = do_batch_transform(
             self.func,
@@ -205,7 +223,7 @@ class BatchTransformStep(ComputeStep):
         for changes in gen:
             pass
 
-    def run_changelist(self, ds: DataStore, change_list: ChangeList, run_config: RunConfig = None) -> ChangeList:
+    def run_changelist(self, ds: DataStore, change_list: ChangeList, run_config: Optional[RunConfig] = None) -> ChangeList:
         run_config = RunConfig.add_labels(run_config, {'step_name': self.name})
 
         idx_count, idx_gen = ds.get_change_list_process_ids(
@@ -235,10 +253,7 @@ class BatchTransformStep(ComputeStep):
         return res_changelist
 
 
-class BatchGenerateFunc(Protocol):
-    # __name__: str
-    def __call__(self, **kwargs) -> Iterator[Union[DataDF, Tuple[DataDF, ...]]]:
-        ...
+BatchGenerateFunc = Callable[..., Iterator[Union[DataDF, Tuple[DataDF, ...]]]]
 
 
 def do_batch_generate(
@@ -246,8 +261,8 @@ def do_batch_generate(
 
     ds: DataStore,
     output_dts: List[DataTable],
-    kwargs: Dict[str, Any] = None,
-    run_config: RunConfig = None,
+    kwargs: Optional[Dict[str, Any]] = None,
+    run_config: Optional[RunConfig] = None,
 ) -> None:
     import inspect
 
@@ -340,8 +355,8 @@ class BatchGenerate(PipelineStep):
 
         return [
             DatatableTransformStep(
-                name=self.func.__name__,  # type: ignore # mypy bug: https://github.com/python/mypy/issues/10976
-                func=transform_func,
+                name=self.func.__name__,
+                func=cast(DatatableTransformFunc, transform_func),
                 input_dts=[],
                 output_dts=[catalog.get_datatable(ds, name) for name in self.outputs],
                 check_for_changes=False,
@@ -350,7 +365,7 @@ class BatchGenerate(PipelineStep):
         ]
 
 
-def update_external_table(ds: DataStore, table: DataTable, run_config: RunConfig = None) -> None:
+def update_external_table(ds: DataStore, table: DataTable, run_config: Optional[RunConfig] = None) -> None:
     now = time.time()
 
     for ps_df in tqdm.tqdm(table.table_store.read_rows_meta_pseudo_df(run_config=run_config)):
@@ -407,7 +422,7 @@ class UpdateExternalTable(PipelineStep):
         return [
             DatatableTransformStep(
                 name=f'update_{self.output_table_name}',
-                func=transform_func,
+                func=cast(DatatableTransformFunc, transform_func),
                 input_dts=[],
                 output_dts=[catalog.get_datatable(ds, self.output_table_name)],
             )
